const cluster = require('cluster');
const express = require('express');

function noop() {}

class GracefulServer {

  constructor(options) {
    options = options || {};

    this.minBounds = options.minBounds || 100;
    this.maxBounds = options.maxBounds || 200;
    this.beforeMiddleware = options.beforeMiddleware || noop;
    this.afterMiddleware = options.afterMiddleware || noop;
    this.app = express();
  }

  serve(fastbootMiddleware) {
    let app = this.app;
    let requestCount = 0;

    let maxRequestCount = this.getMaxRequests();


    this.beforeMiddleware(app);

    app.get('/*', (req, res, next) => {
      requestCount++;
      this.ui.writeLine(`Request ${requestCount}/${maxRequestCount}: ${req.headers['user-agent']}`);
      fastbootMiddleware(req, res, next);
      if (requestCount === maxRequestCount) {
        this.ui.writeLine('reached max request count');
        this.gracefulKill();
      }
    });

    this.afterMiddleware(app);

    return new Promise(resolve => {
      let listener = app.listen(process.env.PORT || 3000, () => {
        let host = listener.address().address;
        let port = listener.address().port;

        this.ui.writeLine('HTTP server started; max requests=%s url=http://%s:%s', maxRequestCount, host, port);
        this.server = listener;
        resolve();
      });
    });
  }

   /*
    We generate a random maximum request count between
    a given bounds. This is because the servers are used
    fairly evenly, and we dont' want them going down
    at the same time. Could be smarter about this, but
    seems to do the job.
    */
  getMaxRequests() {
    let minBounds = this.minBounds;
    let maxBounds = this.maxBounds;
    let flexRoom = maxBounds - minBounds;
    return minBounds + Math.floor(Math.random() * flexRoom);
  }

  gracefulKill() {
    this.ui.writeLine("Received kill signal, shutting down gracefully.");
    process.disconnect();
    this.server.close(() => {
      this.ui.writeLine("Closed out remaining connections.");
      process.exit()
    });

    // kill after a minute just in case processes don't finish in time
    setTimeout(function() {
        console.error("Could not close connections in time, forcefully shutting down");
        process.exit()
    }, 30*1000);
  }
}

module.exports = GracefulServer;
